"""MCP Security Audit — Static security scanner for MCP servers.

Zero-dependency scanner that detects security vulnerabilities in Python,
TypeScript/JavaScript, and Go MCP server source code using AST analysis
and pattern matching. Includes dependency scanning, CI/CD integration,
plugin system, and live runtime scanning.

No Docker, no API keys, no external dependencies. Just Python 3.11+ stdlib.
"""

__version__ = "2.9.0"
__author__ = "Carlos Miret"
__license__ = "MIT"

from .deobfuscate import DeobfuscationFinding, deobfuscate
from .diff import DiffResult, ScanDiff, ScoreDelta
from .flows import FlowAnalyzer, FlowFinding, FlowResult
from .guard import ConfigGuard, ConfigGuardResult
from .live import LiveScanResult, MCPLiveScanner
from .models import Finding, ScanResult, ScanSummary, Severity
from .probes import Probe, ProbeResult, evaluate_response, generate_probe_report
from .probes import get_all_probes as get_injection_probes
from .probes import get_probes_by_category
from .multi import FleetGrade, FleetReport, MultiScanner, detect_mcp_servers
from .plugins import PluginRegistry, PluginRule
from .policy import Policy, PolicyResult, evaluate_policy, load_policy
from .scanner import MCPSecurityScanner, detect_language
from .summary import SummaryReport, build_summary
from .trust_score import TrustScoreResult, compute_trust_score, compute_summary_trust_score
from .fingerprint import (
    FrameworkInfo,
    MCPFingerprint,
    fingerprint_server,
    fingerprint_file,
    fingerprint_path,
)
from .fleet_report import generate_fleet_report, generate_fleet_report_from_dict
from .summary_dashboard import generate_dashboard, generate_dashboard_from_dict
from .rule_gen import (
    GeneratedRule as RuleGenResult,
    RuleTemplate as RuleGenTemplate,
    generate_rule_from_finding,
    generate_rule_from_template,
)
from .rule_generator import (
    GeneratedRule,
    RuleTemplate,
    generate_rules_from_cve,
    generate_rules_from_owasp,
    get_all_generated_rules,
)
from .ts_scanner import TSSecurityScanner
from .go_scanner import GoSecurityScanner
from .dep_scanner import DependencyScanner, DepScanResult, VulnerableDep
from .ci_configs import CIOptions, generate_github_action, generate_gitlab_ci, generate_pre_commit, generate_jenkins
from .fixer import FixStats, compute_fix_stats, format_fix_stats, get_fixable_rule_ids
from .profiles import ComplianceProfile, get_profile, list_profiles, PROFILES
from .sbom import (
    MCPSbom,
    RiskFactor,
    generate_sbom,
    export_sbom_spdx,
    export_sbom_cyclonedx,
    export_sbom_text,
    export_sbom_json,
)
from .playbooks import (
    Playbook,
    PlaybookLibrary,
    RemediationStep,
    generate_playbook,
    generate_playbooks,
    generate_all_playbooks,
    generate_playbook_by_id,
    generate_playbooks_from_findings,
    generate_playbooks_from_scan_data,
    get_covered_rule_ids,
    export_playbooks_html,
    export_playbooks_markdown,
    export_playbooks_json,
)
from .regression import (
    Regression,
    RegressionResult,
    RegressionFinding,
    detect_regressions,
    detect_regressions_from_files,
    detect_regressions_from_history,
    export_regression_html,
    format_regression_report,
    format_regression_text,
    format_regression_json,
    EXIT_CODE_REGRESSION,
)
from .secret_scanner import (
    SecretFinding,
    SecretPattern,
    calculate_entropy,
    get_secret_patterns,
    scan_secrets,
    scan_secrets_file,
    scan_secrets_directory,
    format_secrets_text,
    format_secrets_json,
)
from .sarif_ci import (
    generate_sarif_upload,
    generate_github_annotation,
    generate_gitlab_sast,
    format_sarif_summary,
    export_sarif_json,
    export_github_annotations,
    export_gitlab_sast_json,
)
from .rule_coverage import (
    CoverageGap,
    CoverageMatrix,
    analyze_owasp_coverage,
    analyze_cwe_coverage,
    analyze_mitre_coverage,
    generate_coverage_html,
    suggest_new_rules,
    format_coverage_text,
    format_coverage_json,
)
from .dep_vuln import (
    DepClassification,
    DepFinding,
    DependencyReport,
    DeprecatedPackage,
    LicenseInfo,
    LicenseRisk,
    RiskLevel,
    RiskyPackage,
    VulnReport,
    VulnerableVersion,
    classify_license,
    format_dep_report_json,
    format_dep_report_text,
    format_vuln_json,
    format_vuln_text,
    get_deprecated_packages,
    get_risk_categories,
    get_risky_packages,
    get_typosquat_targets,
    get_vulnerable_versions,
    scan_dependencies,
    scan_deps,
)
from .permission_analyzer import (
    PermissionCategory,
    PermissionIssue,
    PermissionMatrix,
    PermissionPattern,
    PermissionReport,
    PermissionUsage,
    ToolPermission,
    analyze_permissions,
    format_permission_json,
    format_permission_text,
    get_permission_categories,
    get_permission_patterns,
)
from .baseline_manager import (
    Baseline,
    BaselineDiff,
    BaselineEvolution,
    BaselineFinding,
    BaselineSnapshot,
    DiffFinding,
    ScanFinding as BaselineScanFinding,
    ScanResult as BaselineScanResult,
    compare_to_baseline,
    compute_fingerprint,
    create_baseline as create_advanced_baseline,
    filter_baseline,
    format_baseline_json,
    format_baseline_text,
    format_diff_json as format_baseline_diff_json,
    format_diff_text as format_baseline_diff_text,
    format_evolution_json,
    format_evolution_text,
    merge_baselines,
    update_baseline,
)
from .config_drift import (
    ConfigSnapshot,
    DriftItem,
    DriftReport,
    DriftSeverity,
    check_drift,
    check_drift_from_file,
    create_snapshot,
    create_snapshot_from_dict,
    format_drift_json,
    format_drift_text,
    load_snapshot,
    save_snapshot,
)
from .attack_path import (
    AttackEdge,
    AttackGraph,
    AttackNode,
    AttackPath,
    AttackPattern,
    AttackRisk,
    analyze_attack_paths,
    format_attack_dot,
    format_attack_json,
    format_attack_text,
    get_attack_patterns,
)
from .supply_chain import (
    SupplyChainFinding,
    SupplyChainResult,
    SupplyChainScanner,
    detect_typosquat,
    format_supply_chain_html,
    format_supply_chain_json,
    format_supply_chain_text,
)
from .scorecard import (
    ScorecardData,
    build_scorecard_data,
    build_scorecard_from_summary,
    generate_scorecard,
    generate_scorecard_from_summary,
    generate_scorecard_html,
    format_scorecard_json,
)
from .ci_templates import (
    CITemplateOptions,
    generate_ci_template,
    generate_github_actions,
    generate_gitlab_ci as generate_gitlab_ci_template,
    generate_pre_commit as generate_pre_commit_template,
    get_default_output_filename,
    format_ci_template_json,
)
from .protocol_validator import (
    ProtocolIssue,
    ProtocolValidationResult,
    validate_protocol,
    validate_protocol_file,
    validate_protocol_directory,
    validate_tool,
    validate_resource,
    validate_prompt,
    format_protocol_text,
    format_protocol_json,
)
from .secrets_detector import (
    DetectedSecret,
    SecretsReport,
    SecretPatternDef,
    detect_secrets,
    detect_secrets_file,
    detect_secrets_directory,
    get_all_patterns as get_detector_patterns,
    load_allowlist,
    format_secrets_text as format_detector_text,
    format_secrets_json as format_detector_json,
)
from .benchmark import (
    BenchmarkResult,
    FileTiming,
    LanguageTiming,
    MemoryProfile,
    RuleTiming,
    run_benchmark,
    format_benchmark_text,
    format_benchmark_json,
    format_benchmark_html,
)
from .threat_intel import (
    ThreatEntry,
    ThreatIntelDB,
    ThreatMatch,
    ThreatReport,
    ThreatSeverity,
    format_threat_json,
    format_threat_text,
    format_threats_list_text,
    generate_threat_report,
    match_threats,
    match_threats_from_scan,
)
from .compliance_validator import (
    ComplianceCheck,
    ComplianceFramework,
    ComplianceResult,
    ControlResult,
    ControlStatus,
    format_compliance_html,
    format_compliance_json,
    format_compliance_text,
    get_framework_checks,
    list_frameworks,
    validate_compliance,
    validate_compliance_from_file,
)
from .scan_cache import (
    CacheEntry,
    CacheStats,
    IncrementalResult,
    IncrementalScanner,
    ScanCache,
    format_cache_stats_json,
    format_cache_stats_text,
    format_incremental_text,
)
from .risk_scoring import (
    DimensionScore,
    RiskDimension,
    RiskProfile,
    RiskTrend,
    classify_finding_dimension,
    compute_risk_from_scan,
    compute_risk_profile,
    format_risk_html,
    format_risk_json,
    format_risk_text,
    load_weights,
    save_history as save_risk_history,
    load_history as load_risk_history,
    validate_weights,
)
from .report_aggregator import (
    AggregatedFinding,
    AggregatedReport,
    TimelineEntry,
    TrendAnalysis,
    compute_trend_from_reports,
    format_aggregate_html,
    format_aggregate_json,
    format_aggregate_text,
    merge_report_files,
    merge_reports,
)
from .rule_impact import (
    ImpactReport,
    RuleHit,
    RuleImpact,
    analyze_rule_impact,
    analyze_rule_impact_from_file,
    analyze_rule_impact_from_scan,
    compute_effectiveness,
    estimate_fp_rate,
    format_impact_json,
    format_impact_text,
    recommend_action,
)
from .hardener import (
    HardeningChange,
    HardeningResult,
    HardeningRule,
    format_diff as format_hardening_diff,
    format_hardened_json,
    format_hardening_report_json,
    format_hardening_text,
    get_hardening_rules,
    harden_config,
    harden_file,
)
from .audit_trail import (
    AuditEntry,
    AuditLog,
    append_to_log,
    create_entry as create_audit_entry,
    create_fix_entry,
    create_guard_entry,
    create_scan_entry,
    export_csv as export_audit_csv,
    export_html as export_audit_html,
    export_jsonl as export_audit_jsonl,
    format_audit_json,
    format_audit_text,
    load_log as load_audit_log,
    save_log as save_audit_log,
)
from .security_metrics import (
    Alert as MetricsAlert,
    AlertRule as MetricsAlertRule,
    MetricCollector,
    MetricPoint,
    MetricsReport,
    MetricsSnapshot,
    TimeSeries,
    format_metrics_html,
    format_metrics_json,
    format_metrics_text,
    load_metrics,
    save_metrics,
)
from .dashboard_gen import (
    DashboardConfig,
    DashboardData,
    build_dashboard_data,
    format_dashboard_json,
    format_dashboard_text,
    generate_security_dashboard,
    generate_dashboard_from_scan_data,
)
from .scan_profiles import (
    ScanProfile,
    ProfileRegistry,
    get_profile as get_scan_profile,
    list_scan_profiles,
    list_profile_names,
    register_profile,
    load_profile_from_file,
    get_registry as get_profile_registry,
    apply_profile_filters,
    apply_profile_to_rules as apply_scan_profile_to_rules,
    format_profiles_text as format_scan_profiles_text,
    format_profiles_json as format_scan_profiles_json,
    validate_profile as validate_scan_profile,
    PROFILE_QUICK,
    PROFILE_FULL,
    PROFILE_CI,
    PROFILE_COMPLIANCE,
    PROFILE_DEV,
)
from .remediation_engine import (
    FixType,
    RemediationItem,
    RemediationPlan,
    RemediationPriority,
    RemediationStep as RemediationEngineStep,
    build_remediation_item,
    format_remediation_html,
    format_remediation_json,
    format_remediation_text,
    generate_plan_from_dict,
    generate_plan_from_findings,
    generate_remediation_plan,
    get_fix_types,
    get_known_rules,
    get_remediation_info,
)
from .interactive_report import (
    InteractiveReportConfig,
    ReportFinding,
    ReportMetadata,
    extract_findings as extract_report_findings,
    extract_metadata as extract_report_metadata,
    format_interactive_json,
    generate_interactive_report,
    generate_interactive_report_from_scan,
)
from .severity_config import (
    OverrideResult,
    RuleOverride,
    SeverityConfig,
    apply_severity_overrides,
    apply_severity_overrides_detailed,
    create_default_rc,
    format_config_json as format_severity_config_json,
    format_config_text as format_severity_config_text,
    load_severity_config,
    merge_configs as merge_severity_configs,
    parse_severity_config,
    validate_config as validate_severity_config,
)
from .scan_compare import (
    ComparedFinding,
    ComparisonReport,
    SeverityDelta,
    TimelinePoint,
    TimelineReport,
    build_timeline,
    compare_scan_files,
    compare_scans,
    detect_regressions_from_history as detect_scan_regressions,
    format_comparison_json,
    format_comparison_text,
    generate_comparison_html,
)
from .rule_sdk import (
    Rule as SDKRule,
    RuleFinding as SDKRuleFinding,
    RuleMetadata as SDKRuleMetadata,
    RuleRegistry as SDKRuleRegistry,
    create_ast_rule,
    create_regex_rule,
    create_rule as create_sdk_rule,
    format_findings_json as format_sdk_findings_json,
    format_findings_text as format_sdk_findings_text,
    format_registry_text as format_sdk_registry_text,
    get_example_registry,
    rule as rule_decorator,
    validate_rule_metadata,
)
from .multi_project import (
    AggregatedReport as MultiProjectReport,
    FleetStats,
    ProjectFinding,
    ProjectResult,
    format_fleet_json as format_multi_fleet_json,
    format_fleet_text as format_multi_fleet_text,
    generate_fleet_html as generate_multi_fleet_html,
    scan_projects,
    scan_projects_from_parent,
)
from .annotator import (
    Annotation,
    AnnotationStore,
    annotate_finding,
    bulk_annotate,
    filter_annotated,
    filter_by_status as filter_findings_by_status,
    format_annotations_json,
    format_annotations_text,
    load_and_annotate,
    load_annotations,
    make_finding_id,
    make_finding_id_from_dict,
    remove_annotation,
    save_annotations,
)
from .report_generator import (
    ExecutiveReport,
    ReportOptions,
    ReportSection,
    build_executive_report,
    format_report_json,
    format_report_text,
    generate_executive_report,
)
from .registry_scanner import (
    MCPCategory as MCPRegistryCategory,
    MCPEntry,
    RegistryReport,
    RiskAssessment as RegistryRiskAssessment,
    RiskLevel as RegistryRiskLevel,
    assess_mcp_risk,
    format_assessment_text,
    format_mcps_list_text,
    format_registry_json,
    format_registry_text,
    get_known_mcp,
    get_known_mcps,
    load_registry,
    save_registry,
    scan_registry,
)
from .exemptions import (
    AuditEvent as ExemptionAuditEvent,
    Exemption,
    ExemptionStore,
    FilteredFindings,
    apply_exemptions,
    create_exemption,
    expire_exemptions,
    export_exemptions,
    format_audit_trail_text as format_exemption_audit_text,
    format_exemptions_json,
    format_exemptions_text,
    format_filtered_text,
    get_audit_trail as get_exemption_audit_trail,
    import_exemptions,
    load_exemptions,
    revoke_exemption,
    save_exemptions,
    update_exemption,
)
from .threat_model import (
    Asset as ThreatAsset,
    AssetType,
    AttackVector as ThreatAttackVector,
    Impact as ThreatImpact,
    Likelihood,
    Mitigation as ThreatMitigation,
    StrideCategory,
    Threat,
    ThreatAgent,
    ThreatModel,
    format_threat_model_html,
    format_threat_model_json,
    format_threat_model_text,
    generate_threat_model,
)
from .compliance_gaps import (
    Control as ComplianceControl,
    ControlResult as ComplianceGapControlResult,
    ControlStatus as ComplianceGapControlStatus,
    Framework as ComplianceGapFramework,
    FrameworkResult,
    GapAnalysis,
    RemediationAction,
    RemediationPriority as GapRemediationPriority,
    Roadmap,
    analyze_gaps,
    format_gaps_html,
    format_gaps_json,
    format_gaps_text,
    generate_remediation_roadmap,
    get_framework_controls,
    list_supported_frameworks,
)
from .history_analytics import (
    CategoryTrend,
    FindingRecord,
    HistoryAnalytics,
    Prediction,
    analyze_history,
    format_analytics_html,
    format_analytics_json,
    format_analytics_text,
    predict_target_date,
)
from .attack_simulator import (
    AttackCategory,
    AttackPayload,
    AttackScenario,
    AttackSimulation,
    DetectionMethod as AttackDetectionMethod,
    ImpactLevel as AttackImpactLevel,
    Mitigation as AttackMitigation,
    Probability as AttackProbability,
    RiskMatrixCell,
    format_attack_simulation_json,
    format_attack_simulation_text,
    generate_attack_report,
    simulate_attacks,
)
from .posture_score import (
    DimensionResult,
    PostureDimension,
    PostureScore,
    TrendPoint as PostureTrendPoint,
    calculate_posture,
    classify_dimension,
    format_posture_html,
    format_posture_json,
    format_posture_text,
)
from .server_profiler import (
    FrameworkDetection,
    ProfileComparison,
    ProfileDiff,
    PromptInfo,
    ResourceInfo as ProfileResourceInfo,
    ServerProfile,
    ToolInfo,
    TransportInfo,
    compare_profiles,
    format_comparison_json as format_profile_comparison_json,
    format_comparison_text as format_profile_comparison_text,
    format_profile_json,
    format_profile_text,
    profile_server,
)
from .incident_response import (
    ChecklistItem,
    IncidentClassification,
    IncidentPhase,
    IncidentPlaybook,
    IncidentType,
    PlaybookStep as IRPlaybookStep,
    ResponseAction,
    RunbookEntry,
    classify_incident,
    format_playbook_html as format_ir_playbook_html,
    format_playbook_json as format_ir_playbook_json,
    format_playbook_text as format_ir_playbook_text,
    generate_playbook as generate_ir_playbook,
    generate_playbooks_from_findings as generate_ir_from_findings,
    generate_playbooks_from_scan as generate_ir_from_scan,
    generate_runbook,
    get_incident_templates,
    get_template_for_type,
)
from .training_v2 import (
    Answer as TrainingAnswer,
    Curriculum,
    CurriculumModule,
    Question as TrainingQuestion,
    Quiz,
    TrainingLevel,
    TrainingMaterial,
    TrainingModule,
    TrainingModuleType,
    TrainingSection,
    format_curriculum_json,
    format_curriculum_text,
    format_quiz_json,
    format_quiz_text,
    format_training_html,
    format_training_json,
    format_training_text,
    generate_curriculum,
    generate_quiz,
    generate_training,
    get_module_types,
)
from .api_server import (
    ApiResponse,
    McpauditHandler,
    McpauditServer,
    ScanRequest,
    ScanResponse,
    create_api_server,
    format_error_response,
    format_success_response,
    get_routes,
    handle_scan_request,
    parse_scan_request,
)
from .vuln_db_v2 import (
    EnrichedFinding,
    VulnDatabase,
    VulnEntry,
    VulnSearchResult,
    VulnSeverity,
    export_db_json as export_vuln_db_json,
    format_enriched_json,
    format_enriched_text,
    format_vuln_db_json,
    format_vuln_db_text,
    import_db_json as import_vuln_db_json,
    match_findings as match_vuln_findings,
)
from .code_quality import (
    DeadCodePattern,
    DuplicateBlock,
    FunctionMetrics,
    QualityIssue,
    QualityLevel,
    QualityReport,
    analyze_code_quality,
    format_quality_json,
    format_quality_text,
)
from .test_generator import (
    DetectedPrompt as TGDetectedPrompt,
    DetectedResource as TGDetectedResource,
    DetectedTool as TGDetectedTool,
    GeneratedTest,
    ServerProfile as TGServerProfile,
    GeneratedSuite,
    detect_server_components,
    format_suite_json,
    format_suite_text,
    generate_test_suite,
    generate_tests,
)
from .metrics_dashboard import (
    DashboardOptions as MetricsDashboardOptions,
    DashboardResult as MetricsDashboardResult,
    KPI as MetricsKPI,
    ScanEntry as MetricsScanEntry,
    compute_kpis as compute_metrics_kpis,
    format_dashboard_result_json as format_metrics_dashboard_json,
    generate_metrics_dashboard,
)
from .tool_perm_mapper import (
    APICategory,
    APIPattern as ToolAPIPattern,
    APIUsage,
    Recommendation as ToolRecommendation,
    ToolMapping,
    ToolPermissionMap,
    format_perm_map_json,
    format_perm_map_text,
    format_recommendations_json as format_tool_recommendations_json,
    format_recommendations_text as format_tool_recommendations_text,
    get_api_categories,
    get_api_patterns,
    map_tool_permissions,
    recommend_permissions,
)
from .compliance_cert import (
    AuditorInfo,
    CertificateOptions,
    CertificateResult,
    FrameworkCompliance,
    ScanResultSummary as CertScanResult,
    format_certificate_json,
    generate_certificate,
    generate_certificate_with_metadata,
)

__all__ = [
    "ConfigGuard",
    "ConfigGuardResult",
    "DeobfuscationFinding",
    "DiffResult",
    "FileWatcher",
    "Finding",
    "FixResult",
    "FixSuggestion",
    "FleetGrade",
    "FleetReport",
    "FrameworkInfo",
    "GeneratedRule",
    "FlowAnalyzer",
    "FlowFinding",
    "FlowResult",
    "LiveScanResult",
    "MCPLiveScanner",
    "MCPSecurityScanner",
    "MultiScanner",
    "PluginRegistry",
    "PluginRule",
    "Policy",
    "Probe",
    "ProbeResult",
    "PolicyResult",
    "ScanDiff",
    "ScanResult",
    "ScanSummary",
    "ScoreDelta",
    "Severity",
    "SummaryReport",
    "TSSecurityScanner",
    "GoSecurityScanner",
    "DependencyScanner",
    "DepScanResult",
    "VulnerableDep",
    "CIOptions",
    "TrustScoreResult",
    "RuleTemplate",
    "build_summary",
    "compute_summary_trust_score",
    "compute_trust_score",
    "deobfuscate",
    "detect_language",
    "MCPFingerprint",
    "RuleGenResult",
    "RuleGenTemplate",
    "fingerprint_file",
    "fingerprint_path",
    "fingerprint_server",
    "generate_dashboard",
    "generate_dashboard_from_dict",
    "generate_fleet_report",
    "generate_fleet_report_from_dict",
    "generate_rule_from_finding",
    "generate_rule_from_template",
    "generate_rules_from_cve",
    "generate_rules_from_owasp",
    "get_all_generated_rules",
    "detect_mcp_servers",
    "evaluate_policy",
    "evaluate_response",
    "generate_probe_report",
    "get_injection_probes",
    "get_probes_by_category",
    "load_policy",
    "generate_github_action",
    "generate_gitlab_ci",
    "generate_pre_commit",
    "generate_jenkins",
    "ComplianceProfile",
    "FixStats",
    "compute_fix_stats",
    "format_fix_stats",
    "get_fixable_rule_ids",
    "get_profile",
    "list_profiles",
    "PROFILES",
    "MCPSbom",
    "RiskFactor",
    "generate_sbom",
    "export_sbom_spdx",
    "export_sbom_cyclonedx",
    "export_sbom_text",
    "export_sbom_json",
    "Playbook",
    "PlaybookLibrary",
    "RemediationStep",
    "generate_playbook",
    "generate_playbooks",
    "generate_all_playbooks",
    "generate_playbook_by_id",
    "generate_playbooks_from_findings",
    "generate_playbooks_from_scan_data",
    "get_covered_rule_ids",
    "export_playbooks_html",
    "export_playbooks_markdown",
    "export_playbooks_json",
    "Regression",
    "RegressionResult",
    "RegressionFinding",
    "detect_regressions",
    "detect_regressions_from_files",
    "detect_regressions_from_history",
    "export_regression_html",
    "format_regression_report",
    "format_regression_text",
    "format_regression_json",
    "EXIT_CODE_REGRESSION",
    # secret_scanner
    "SecretFinding",
    "SecretPattern",
    "calculate_entropy",
    "get_secret_patterns",
    "scan_secrets",
    "scan_secrets_file",
    "scan_secrets_directory",
    "format_secrets_text",
    "format_secrets_json",
    # sarif_ci
    "generate_sarif_upload",
    "generate_github_annotation",
    "generate_gitlab_sast",
    "format_sarif_summary",
    "export_sarif_json",
    "export_github_annotations",
    "export_gitlab_sast_json",
    # rule_coverage
    "CoverageGap",
    "CoverageMatrix",
    "analyze_owasp_coverage",
    "analyze_cwe_coverage",
    "analyze_mitre_coverage",
    "generate_coverage_html",
    "suggest_new_rules",
    "format_coverage_text",
    "format_coverage_json",
    # dep_vuln
    "DepClassification",
    "RiskLevel",
    "RiskyPackage",
    "VulnReport",
    "format_vuln_json",
    "format_vuln_text",
    "get_risk_categories",
    "get_risky_packages",
    "scan_deps",
    # config_drift
    "ConfigSnapshot",
    "DriftItem",
    "DriftReport",
    "DriftSeverity",
    "check_drift",
    "check_drift_from_file",
    "create_snapshot",
    "create_snapshot_from_dict",
    "format_drift_json",
    "format_drift_text",
    "load_snapshot",
    "save_snapshot",
    # attack_path
    "AttackEdge",
    "AttackGraph",
    "AttackNode",
    "AttackPath",
    "AttackPattern",
    "AttackRisk",
    "analyze_attack_paths",
    "format_attack_dot",
    "format_attack_json",
    "format_attack_text",
    "get_attack_patterns",
    # supply_chain
    "SupplyChainFinding",
    "SupplyChainResult",
    "SupplyChainScanner",
    "detect_typosquat",
    "format_supply_chain_html",
    "format_supply_chain_json",
    "format_supply_chain_text",
    # scorecard
    "ScorecardData",
    "build_scorecard_data",
    "build_scorecard_from_summary",
    "generate_scorecard",
    "generate_scorecard_from_summary",
    "generate_scorecard_html",
    "format_scorecard_json",
    # ci_templates
    "CITemplateOptions",
    "generate_ci_template",
    "generate_github_actions",
    "generate_gitlab_ci_template",
    "generate_pre_commit_template",
    "get_default_output_filename",
    "format_ci_template_json",
    # protocol_validator
    "ProtocolIssue",
    "ProtocolValidationResult",
    "validate_protocol",
    "validate_protocol_file",
    "validate_protocol_directory",
    "validate_tool",
    "validate_resource",
    "validate_prompt",
    "format_protocol_text",
    "format_protocol_json",
    # secrets_detector
    "DetectedSecret",
    "SecretsReport",
    "SecretPatternDef",
    "detect_secrets",
    "detect_secrets_file",
    "detect_secrets_directory",
    "get_detector_patterns",
    "load_allowlist",
    "format_detector_text",
    "format_detector_json",
    # benchmark
    "BenchmarkResult",
    "FileTiming",
    "LanguageTiming",
    "MemoryProfile",
    "RuleTiming",
    "run_benchmark",
    "format_benchmark_text",
    "format_benchmark_json",
    "format_benchmark_html",
    # threat_intel
    "ThreatEntry",
    "ThreatIntelDB",
    "ThreatMatch",
    "ThreatReport",
    "ThreatSeverity",
    "format_threat_json",
    "format_threat_text",
    "format_threats_list_text",
    "generate_threat_report",
    "match_threats",
    "match_threats_from_scan",
    # compliance_validator
    "ComplianceCheck",
    "ComplianceFramework",
    "ComplianceResult",
    "ControlResult",
    "ControlStatus",
    "format_compliance_html",
    "format_compliance_json",
    "format_compliance_text",
    "get_framework_checks",
    "list_frameworks",
    "validate_compliance",
    "validate_compliance_from_file",
    # scan_cache
    "CacheEntry",
    "CacheStats",
    "IncrementalResult",
    "IncrementalScanner",
    "ScanCache",
    "format_cache_stats_json",
    "format_cache_stats_text",
    "format_incremental_text",
    # risk_scoring
    "DimensionScore",
    "RiskDimension",
    "RiskProfile",
    "RiskTrend",
    "classify_finding_dimension",
    "compute_risk_from_scan",
    "compute_risk_profile",
    "format_risk_html",
    "format_risk_json",
    "format_risk_text",
    "load_weights",
    "save_risk_history",
    "load_risk_history",
    "validate_weights",
    # report_aggregator
    "AggregatedFinding",
    "AggregatedReport",
    "TimelineEntry",
    "TrendAnalysis",
    "compute_trend_from_reports",
    "format_aggregate_html",
    "format_aggregate_json",
    "format_aggregate_text",
    "merge_report_files",
    "merge_reports",
    # rule_impact
    "ImpactReport",
    "RuleHit",
    "RuleImpact",
    "analyze_rule_impact",
    "analyze_rule_impact_from_file",
    "analyze_rule_impact_from_scan",
    "compute_effectiveness",
    "estimate_fp_rate",
    "format_impact_json",
    "format_impact_text",
    "recommend_action",
    # hardener
    "HardeningChange",
    "HardeningResult",
    "HardeningRule",
    "format_hardening_diff",
    "format_hardened_json",
    "format_hardening_report_json",
    "format_hardening_text",
    "get_hardening_rules",
    "harden_config",
    "harden_file",
    # audit_trail
    "AuditEntry",
    "AuditLog",
    "append_to_log",
    "create_audit_entry",
    "create_fix_entry",
    "create_guard_entry",
    "create_scan_entry",
    "export_audit_csv",
    "export_audit_html",
    "export_audit_jsonl",
    "format_audit_json",
    "format_audit_text",
    "load_audit_log",
    "save_audit_log",
    # security_metrics
    "MetricsAlert",
    "MetricsAlertRule",
    "MetricCollector",
    "MetricPoint",
    "MetricsReport",
    "MetricsSnapshot",
    "TimeSeries",
    "format_metrics_html",
    "format_metrics_json",
    "format_metrics_text",
    "load_metrics",
    "save_metrics",
    # dashboard_gen
    "DashboardConfig",
    "DashboardData",
    "build_dashboard_data",
    "format_dashboard_json",
    "format_dashboard_text",
    "generate_security_dashboard",
    "generate_dashboard_from_scan_data",
    # scan_profiles
    "ScanProfile",
    "ProfileRegistry",
    "get_scan_profile",
    "list_scan_profiles",
    "list_profile_names",
    "register_profile",
    "load_profile_from_file",
    "get_profile_registry",
    "apply_profile_filters",
    "apply_scan_profile_to_rules",
    "format_scan_profiles_text",
    "format_scan_profiles_json",
    "validate_scan_profile",
    "PROFILE_QUICK",
    "PROFILE_FULL",
    "PROFILE_CI",
    "PROFILE_COMPLIANCE",
    "PROFILE_DEV",
    # remediation_engine
    "FixType",
    "RemediationItem",
    "RemediationPlan",
    "RemediationPriority",
    "RemediationEngineStep",
    "build_remediation_item",
    "format_remediation_html",
    "format_remediation_json",
    "format_remediation_text",
    "generate_plan_from_dict",
    "generate_plan_from_findings",
    "generate_remediation_plan",
    "get_fix_types",
    "get_known_rules",
    "get_remediation_info",
    # interactive_report
    "InteractiveReportConfig",
    "ReportFinding",
    "ReportMetadata",
    "extract_report_findings",
    "extract_report_metadata",
    "format_interactive_json",
    "generate_interactive_report",
    "generate_interactive_report_from_scan",
    # severity_config
    "OverrideResult",
    "RuleOverride",
    "SeverityConfig",
    "apply_severity_overrides",
    "apply_severity_overrides_detailed",
    "create_default_rc",
    "format_severity_config_json",
    "format_severity_config_text",
    "load_severity_config",
    "merge_severity_configs",
    "parse_severity_config",
    "validate_severity_config",
    # scan_compare
    "ComparedFinding",
    "ComparisonReport",
    "SeverityDelta",
    "TimelinePoint",
    "TimelineReport",
    "build_timeline",
    "compare_scan_files",
    "compare_scans",
    "detect_scan_regressions",
    "format_comparison_json",
    "format_comparison_text",
    "generate_comparison_html",
    # rule_sdk
    "SDKRule",
    "SDKRuleFinding",
    "SDKRuleMetadata",
    "SDKRuleRegistry",
    "create_ast_rule",
    "create_regex_rule",
    "create_sdk_rule",
    "format_sdk_findings_json",
    "format_sdk_findings_text",
    "format_sdk_registry_text",
    "get_example_registry",
    "rule_decorator",
    "validate_rule_metadata",
    # multi_project
    "MultiProjectReport",
    "FleetStats",
    "ProjectFinding",
    "ProjectResult",
    "format_multi_fleet_json",
    "format_multi_fleet_text",
    "generate_multi_fleet_html",
    "scan_projects",
    "scan_projects_from_parent",
    # annotator
    "Annotation",
    "AnnotationStore",
    "annotate_finding",
    "bulk_annotate",
    "filter_annotated",
    "filter_findings_by_status",
    "format_annotations_json",
    "format_annotations_text",
    "load_and_annotate",
    "load_annotations",
    "make_finding_id",
    "make_finding_id_from_dict",
    "remove_annotation",
    "save_annotations",
    # report_generator
    "ExecutiveReport",
    "ReportOptions",
    "ReportSection",
    "build_executive_report",
    "format_report_json",
    "format_report_text",
    "generate_executive_report",
    # registry_scanner
    "MCPRegistryCategory",
    "MCPEntry",
    "RegistryReport",
    "RegistryRiskAssessment",
    "RegistryRiskLevel",
    "assess_mcp_risk",
    "format_assessment_text",
    "format_mcps_list_text",
    "format_registry_json",
    "format_registry_text",
    "get_known_mcp",
    "get_known_mcps",
    "load_registry",
    "save_registry",
    "scan_registry",
    # exemptions
    "ExemptionAuditEvent",
    "Exemption",
    "ExemptionStore",
    "FilteredFindings",
    "apply_exemptions",
    "create_exemption",
    "expire_exemptions",
    "export_exemptions",
    "format_exemption_audit_text",
    "format_exemptions_json",
    "format_exemptions_text",
    "format_filtered_text",
    "get_exemption_audit_trail",
    "import_exemptions",
    "load_exemptions",
    "revoke_exemption",
    "save_exemptions",
    "update_exemption",
    # threat_model
    "ThreatAsset",
    "AssetType",
    "ThreatAttackVector",
    "ThreatImpact",
    "Likelihood",
    "ThreatMitigation",
    "StrideCategory",
    "Threat",
    "ThreatAgent",
    "ThreatModel",
    "format_threat_model_html",
    "format_threat_model_json",
    "format_threat_model_text",
    "generate_threat_model",
    # compliance_gaps
    "ComplianceControl",
    "ComplianceGapControlResult",
    "ComplianceGapControlStatus",
    "ComplianceGapFramework",
    "FrameworkResult",
    "GapAnalysis",
    "RemediationAction",
    "GapRemediationPriority",
    "Roadmap",
    "analyze_gaps",
    "format_gaps_html",
    "format_gaps_json",
    "format_gaps_text",
    "generate_remediation_roadmap",
    "get_framework_controls",
    "list_supported_frameworks",
    # history_analytics
    "CategoryTrend",
    "FindingRecord",
    "HistoryAnalytics",
    "Prediction",
    "analyze_history",
    "format_analytics_html",
    "format_analytics_json",
    "format_analytics_text",
    "predict_target_date",
    # dep_vuln v2.5.0 (enhanced)
    "DepFinding",
    "DependencyReport",
    "DeprecatedPackage",
    "LicenseInfo",
    "LicenseRisk",
    "VulnerableVersion",
    "classify_license",
    "format_dep_report_json",
    "format_dep_report_text",
    "get_deprecated_packages",
    "get_typosquat_targets",
    "get_vulnerable_versions",
    "scan_dependencies",
    # permission_analyzer
    "PermissionCategory",
    "PermissionIssue",
    "PermissionMatrix",
    "PermissionPattern",
    "PermissionReport",
    "PermissionUsage",
    "ToolPermission",
    "analyze_permissions",
    "format_permission_json",
    "format_permission_text",
    "get_permission_categories",
    "get_permission_patterns",
    # baseline_manager
    "Baseline",
    "BaselineDiff",
    "BaselineEvolution",
    "BaselineFinding",
    "BaselineSnapshot",
    "BaselineScanFinding",
    "BaselineScanResult",
    "DiffFinding",
    "compare_to_baseline",
    "compute_fingerprint",
    "create_advanced_baseline",
    "filter_baseline",
    "format_baseline_json",
    "format_baseline_text",
    "format_baseline_diff_json",
    "format_baseline_diff_text",
    "format_evolution_json",
    "format_evolution_text",
    "merge_baselines",
    "update_baseline",
    # attack_simulator v2.6.0
    "AttackCategory",
    "AttackPayload",
    "AttackScenario",
    "AttackSimulation",
    "AttackDetectionMethod",
    "AttackImpactLevel",
    "AttackMitigation",
    "AttackProbability",
    "RiskMatrixCell",
    "format_attack_simulation_json",
    "format_attack_simulation_text",
    "generate_attack_report",
    "simulate_attacks",
    # posture_score v2.6.0
    "DimensionResult",
    "PostureDimension",
    "PostureScore",
    "PostureTrendPoint",
    "calculate_posture",
    "classify_dimension",
    "format_posture_html",
    "format_posture_json",
    "format_posture_text",
    # server_profiler v2.6.0
    "FrameworkDetection",
    "ProfileComparison",
    "ProfileDiff",
    "PromptInfo",
    "ProfileResourceInfo",
    "ServerProfile",
    "ToolInfo",
    "TransportInfo",
    "compare_profiles",
    "format_profile_comparison_json",
    "format_profile_comparison_text",
    "format_profile_json",
    "format_profile_text",
    "profile_server",
    # incident_response v2.7.0
    "ChecklistItem",
    "IncidentClassification",
    "IncidentPhase",
    "IncidentPlaybook",
    "IncidentType",
    "IRPlaybookStep",
    "ResponseAction",
    "RunbookEntry",
    "classify_incident",
    "format_ir_playbook_html",
    "format_ir_playbook_json",
    "format_ir_playbook_text",
    "generate_ir_playbook",
    "generate_ir_from_findings",
    "generate_ir_from_scan",
    "generate_runbook",
    "get_incident_templates",
    "get_template_for_type",
    # training_v2 v2.7.0
    "TrainingAnswer",
    "Curriculum",
    "CurriculumModule",
    "TrainingQuestion",
    "Quiz",
    "TrainingLevel",
    "TrainingMaterial",
    "TrainingModule",
    "TrainingModuleType",
    "TrainingSection",
    "format_curriculum_json",
    "format_curriculum_text",
    "format_quiz_json",
    "format_quiz_text",
    "format_training_html",
    "format_training_json",
    "format_training_text",
    "generate_curriculum",
    "generate_quiz",
    "generate_training",
    "get_module_types",
    # api_server v2.7.0
    "ApiResponse",
    "McpauditHandler",
    "McpauditServer",
    "ScanRequest",
    "ScanResponse",
    "create_api_server",
    "format_error_response",
    "format_success_response",
    "get_routes",
    "handle_scan_request",
    "parse_scan_request",
    # vuln_db_v2 v2.8.0
    "EnrichedFinding",
    "VulnDatabase",
    "VulnEntry",
    "VulnSearchResult",
    "VulnSeverity",
    "export_vuln_db_json",
    "format_enriched_json",
    "format_enriched_text",
    "format_vuln_db_json",
    "format_vuln_db_text",
    "import_vuln_db_json",
    "match_vuln_findings",
    # code_quality v2.8.0
    "DeadCodePattern",
    "DuplicateBlock",
    "FunctionMetrics",
    "QualityIssue",
    "QualityLevel",
    "QualityReport",
    "analyze_code_quality",
    "format_quality_json",
    "format_quality_text",
    # test_generator v2.8.0
    "TGDetectedPrompt",
    "TGDetectedResource",
    "TGDetectedTool",
    "GeneratedTest",
    "TGServerProfile",
    "GeneratedSuite",
    "detect_server_components",
    "format_suite_json",
    "format_suite_text",
    "generate_test_suite",
    "generate_tests",
    # metrics_dashboard v2.9.0
    "MetricsDashboardOptions",
    "MetricsDashboardResult",
    "MetricsKPI",
    "MetricsScanEntry",
    "compute_metrics_kpis",
    "format_metrics_dashboard_json",
    "generate_metrics_dashboard",
    # tool_perm_mapper v2.9.0
    "APICategory",
    "ToolAPIPattern",
    "APIUsage",
    "ToolRecommendation",
    "ToolMapping",
    "ToolPermissionMap",
    "format_perm_map_json",
    "format_perm_map_text",
    "format_tool_recommendations_json",
    "format_tool_recommendations_text",
    "get_api_categories",
    "get_api_patterns",
    "map_tool_permissions",
    "recommend_permissions",
    # compliance_cert v2.9.0
    "AuditorInfo",
    "CertificateOptions",
    "CertificateResult",
    "FrameworkCompliance",
    "CertScanResult",
    "format_certificate_json",
    "generate_certificate",
    "generate_certificate_with_metadata",
]
